/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.btree;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Point;

//FIXME
public class ClusterNode {

	private ClusterNode leftNode;
	private ClusterNode rightNode;

	private ClusterNode parent;

	private Extent extent;
	private Point medoid;

	private double minSphereRadius;

	/**
	 * Creates a ClusterNode where node1 and node2 are nodes or leaves.
	 * 
	 * @param node1
	 * @param node2
	 */
	public ClusterNode(ClusterNode node1, ClusterNode node2) {
		setNodes(node1, node2);
		node1.setParent(this);
		node2.setParent(this);
	}

	public boolean isRoot() {
		return parent == null;
	}

	private void setNodes(ClusterNode node1, ClusterNode node2) {
		double minX, minY, maxX, maxY;
		
		minX = Math.min(node1.getExtent().getMinX(), node2.getExtent().getMinX());
		minY = Math.min(node1.getExtent().getMinY(), node2.getExtent().getMinY());
		maxX = Math.max(node1.getExtent().getMaxX(), node2.getExtent().getMaxX());
		maxY = Math.max(node1.getExtent().getMaxY(), node2.getExtent().getMaxY());
		
		Extent e = new Extent(minX, minY, maxX, maxY);
		if (e.isValid()) {
			this.leftNode = node1;
			this.rightNode = node2;
		} else {
			this.rightNode = node1;
			this.leftNode = node2;
		}
		updateValues();
	}

	/**
	 * Constructs a ClusterNode that in fact is a leaf. So
	 * {@link ClusterNode#leftNode} and {@link ClusterNode#rightNode} will be
	 * null.
	 * 
	 * The {@link ClusterNode#extent} will be a rectangle with area 0 defined by
	 * the Cluster#medoid
	 * 
	 * So the {@link ClusterNode#minSphereRadius} will also be 0
	 * 
	 * @param medoid
	 *            The Point
	 */
	public ClusterNode(Point medoid) {
		this.medoid = medoid;
		updateValues();
	}

	public boolean isChild() {
		return leftNode == null && rightNode == null;
	}

	/**
	 * @return the leftNode
	 */
	public ClusterNode getLeftNode() {
		return leftNode;
	}

	/**
	 * @param leftNode
	 *            the leftNode to set
	 */
	public void setLeftNode(ClusterNode leftNode) {
		Extent e = new Extent(leftNode.getExtent().getLefBottomCoordinate(),
				this.rightNode.getExtent().getRightTopCoordinate());
		if (e.isValid())
			this.leftNode = leftNode;
		else
			this.rightNode = leftNode;
		updateValues();
	}

	/**
	 * @return the rightNode
	 */
	public ClusterNode getRightNode() {
		return rightNode;
	}

	/**
	 * @param rightNode
	 *            the rightNode to set
	 */
	public void setRightNode(ClusterNode rightNode) {
		Extent e = new Extent(leftNode.getExtent().getLefBottomCoordinate(),
				rightNode.getExtent().getRightTopCoordinate());
		if (e.isValid())
			this.rightNode = rightNode;
		else
			this.leftNode = rightNode;
		updateValues();
	}

	/**
	 * @return the extent
	 */
	public Extent getExtent() {
		return extent;
	}

	/**
	 * @param extent
	 *            the extent to set
	 */
	public void setExtent(Extent extent) {
		this.extent = extent;
	}

	/**
	 * @return the medoid
	 */
	public Point getMedoid() {
		return medoid;
	}

	/**
	 * @param medoid
	 *            the medoid to set
	 */
	public void setMedoid(Point medoid) {
		this.medoid = medoid;
	}

	/**
	 * The minimium bounding sphere radius is approximated as the (euclidean)
	 * distance from the center of the bounding box (Extent) of the ClusterNode
	 * to one of its corners (of the Extent)
	 * 
	 * @return the minSphereRadius
	 */
	public double getMinSphereRadius() {
		return this.minSphereRadius;
	}

	private void updateValues() {
		// the cluster is a leaf
		if (leftNode == null && rightNode == null)
			extent = new Extent(medoid, medoid);
		else {
			if (leftNode == null) {
				// has this case sense?
				extent = leftNode.getExtent();
				medoid = leftNode.getMedoid();
			} else if (rightNode == null) {
				// has this case sense?
				extent = rightNode.getExtent();
				medoid = rightNode.getMedoid();
			} else {
				// the cluster is a node
				double minX, minY, maxX, maxY;
				
				minX = Math.min(leftNode.getExtent().getMinX(), rightNode.getExtent().getMinX());
				minY = Math.min(leftNode.getExtent().getMinY(), rightNode.getExtent().getMinY());
				maxX = Math.max(leftNode.getExtent().getMaxX(), rightNode.getExtent().getMaxX());
				maxY = Math.max(leftNode.getExtent().getMaxY(), rightNode.getExtent().getMaxY());
				
				extent = new Extent(minX, minY, maxX, maxY);
				
				Point center = extent.getCenter();
				double dl = leftNode.getMedoid().distance(center);
				double dr = rightNode.getMedoid().distance(center);

				if (dl <= dr)
					medoid = leftNode.getMedoid();
				else if (dl > dr)
					medoid = rightNode.getMedoid();
				// else
				// medoid = center;
			}
		}

		// if Cluster is a leaf the radius will be 0
		this.minSphereRadius = this.extent.getCenter().distance(
				this.extent.getLefBottomCoordinate());
	}

	/**
	 * @return the parent
	 */
	public ClusterNode getParent() {
		return parent;
	}

	/**
	 * @param parent
	 *            the parent to set
	 */
	public void setParent(ClusterNode parent) {
		this.parent = parent;
	}
}
