/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.poi;

import es.prodevelop.gvsig.mini.geom.Extent;

/**
 * Represents a Street on the OSM database. The x and y values represent one of
 * the middle vertexs of the street and the BoundingBox the MBR that contains
 * the street, it can be used to zoom to all the street
 * 
 * @author aromeu
 * 
 */
public class OsmPOIStreet extends POI {

	// private static final Logger logger = LoggerFactory
	// .getLogger(OsmPOIStreet.class);

	private String ref = "";
	private Extent boundingBox = new Extent(0, 0, 0, 0);

	private final static int ID_POS = 0;
	private final static int X_POS = 1;
	private final static int Y_POS = 2;
	private final static int NAME_POS = 3;
	private final static int REF_POS = 4;
	private final static int MINX_POS = 5;
	private final static int MINY_POS = 6;
	private final static int MAXX_POS = 7;
	private final static int MAXY_POS = 8;

	private final static int NUM_PARMS = 9;

	/**
	 * Void constructor for Perst
	 */
	public OsmPOIStreet() {
		super();
	}

	/**
	 * 
	 * @param id
	 *            The ID of the street, usually the primary key on the OSM
	 *            database
	 * @param x
	 *            The x coordinate of one of the middle vertexs of the street
	 * @param y
	 *            The y coordinate of one of the middle vertexs of the street
	 * @param name
	 *            The name of the street
	 * @param ref
	 * @param minX
	 *            The minX of the bounding box that contains the street
	 * @param minY
	 *            The minY of the bounding box that contains the street
	 * @param maxX
	 *            The maxX of the bounding box that contains the street
	 * @param maxY
	 *            The maxY of the bounding box that contains the street
	 */
	public OsmPOIStreet(int id, double x, double y, String name, String ref,
			double minX, double minY, double maxX, double maxY) {
		super(x, y, name);
		this.setId(id);
		this.ref = ref;
		this.boundingBox = new Extent(minX, minY, maxX, maxY);
	}

	public String getRef() {
		return ref;
	}

	public void setRef(String ref) {
		this.ref = ref;
	}

	/**
	 * The bounding box that contains the street
	 * 
	 * @return
	 */
	public Extent getBoundingBox() {
		return boundingBox;
	}

	public void setBoundingBox(Extent boundingBox) {
		this.boundingBox = boundingBox;
	}

	/**
	 * The order of the attributes is: id,x,y,name,ref,minx,miny,maxx,maxy Where
	 * the "," is substituted by @see {@link POI#POI_SEPARATOR}
	 */
	public String toString() {
		return new StringBuffer().append(this.getId())
				.append(POI.POI_SEPARATOR).append(this.getX())
				.append(POI.POI_SEPARATOR).append(this.getY())
				.append(POI.POI_SEPARATOR).append(this.getDescription())
				.append(POI.POI_SEPARATOR).append(ref)
				.append(POI.POI_SEPARATOR).append(this.boundingBox.getMinX())
				.append(POI.POI_SEPARATOR).append(this.boundingBox.getMinY())
				.append(POI.POI_SEPARATOR).append(this.boundingBox.getMaxX())
				.append(POI.POI_SEPARATOR).append(this.boundingBox.getMaxY())
				.toString();
	}

	/**
	 * Parses a line like the one that returns {@link #toString()}
	 * 
	 * @param poiLine
	 * @return
	 */
	public static OsmPOIStreet parse(String poiLine) {
		try {
			String[] parms = poiLine.split(POI.POI_SEPARATOR);

			if (parms == null)
				return null;

			int id = -1;
			double x = 0;
			double y = 0;
			double minX = 0;
			double minY = 0;
			double maxX = 0;
			double maxY = 0;
			String name = null;
			String ref = null;

			try {
				id = Integer.valueOf(parms[ID_POS]).intValue();
			} catch (Exception ignore) {

			}

			try {
				x = Double.valueOf(parms[X_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				y = Double.valueOf(parms[Y_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				name = parms[NAME_POS];
			} catch (Exception ignore) {

			}

			try {
				ref = parms[REF_POS];
			} catch (Exception ignore) {

			}

			try {
				minX = Double.valueOf(parms[MINX_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				maxX = Double.valueOf(parms[MAXX_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				minY = Double.valueOf(parms[MINY_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				maxY = Double.valueOf(parms[MAXY_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			if (name != null && name.length() > 0) {
				OsmPOIStreet poi = new OsmPOIStreet(id, x, y, name, ref, minX,
						minY, maxX, maxY);

				return poi;
			}
			return null;
		} catch (Exception e) {
			// logger.error("parse OsmPOIStreet", e);
			return null;
		}
	}

	public void destroy() {
		ref = null;
		boundingBox = null;
		super.destroy();
	}

	public Object clone() {
		return new OsmPOIStreet(getId(), getX(), getY(), getDescription(), ref,
				this.boundingBox.getMinX(), this.boundingBox.getMinY(),
				this.boundingBox.getMaxX(), this.boundingBox.getMaxY());
	}
}
