/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.persist.perst;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import org.garret.perst.BitIndex;
import org.garret.perst.Blob;
import org.garret.perst.CustomAllocator;
import org.garret.perst.CustomSerializer;
import org.garret.perst.FieldIndex;
import org.garret.perst.IFile;
import org.garret.perst.INamedClassLoader;
import org.garret.perst.IPersistent;
import org.garret.perst.IPersistentHash;
import org.garret.perst.IPersistentList;
import org.garret.perst.IPersistentMap;
import org.garret.perst.IPersistentSet;
import org.garret.perst.Index;
import org.garret.perst.Link;
import org.garret.perst.MultidimensionalComparator;
import org.garret.perst.MultidimensionalIndex;
import org.garret.perst.NullFile;
import org.garret.perst.PatriciaTrie;
import org.garret.perst.PersistentComparator;
import org.garret.perst.Query;
import org.garret.perst.Relation;
import org.garret.perst.SortedCollection;
import org.garret.perst.SpatialIndex;
import org.garret.perst.SpatialIndexR2;
import org.garret.perst.SqlOptimizerParameters;
import org.garret.perst.Storage;
import org.garret.perst.StorageFactory;
import org.garret.perst.StorageListener;
import org.garret.perst.TimeSeries;
import org.garret.perst.XMLImportException;
import org.garret.perst.fulltext.FullTextIndex;
import org.garret.perst.fulltext.FullTextSearchHelper;
import org.garret.perst.impl.ThreadTransactionContext;

/**
 * Contains some facade methods and delegates of Perst Storage class
 * 
 * @author aromeu
 * 
 */
public class PerstDBHelper {

	public Storage db;

	public final static int DEFAULT_PAGE_POOL = 1 * 512 * 512;

	/**
	 * Creates and opens a Perst database with
	 * {@link Storage#INFINITE_PAGE_POOL}
	 * 
	 * @param databaseName
	 *            The name of the database
	 */
	public void openDatabase(String databaseName) {
		db = StorageFactory.getInstance().createStorage();
		db.open(databaseName, Storage.INFINITE_PAGE_POOL);
		db.setProperty("perst.object.cache.init.size", new Integer(100));
	}

	/**
	 * Creates and opens a Perst database with with the specified pagePoolSize
	 * 
	 * @param databaseName
	 *            The name of the database
	 * @param pagePoolSize
	 *            The page pool is used by Perst to keep the most frequently
	 *            used database pages in memory, which reduces disk IO and
	 *            improves performance. Generally, a larger page pool leads to
	 *            faster execution. However, memory is also needed for other
	 *            applications and for the operating system; if you specify a
	 *            very large page pool and it doesn't fit in main memory, it
	 *            will be swapped to disk and performance will degrade.
	 *            Therefore, system resources must be considered when specifying
	 *            the page pool size.
	 */
	public void openDatabase(String databaseName, int pagePoolSize) {
		db = StorageFactory.getInstance().createStorage();
		db.open(databaseName, pagePoolSize);
		db.setProperty("perst.object.cache.init.size", new Integer(200));
	}

	/**
	 * Creates and open a in-memory database with
	 * {@link Storage#INFINITE_PAGE_POOL} pagePoolSize
	 */
	public void openMemoryDatabase() {
		db = StorageFactory.getInstance().createStorage();
		db.open(new NullFile(), // Dummy implementation of IFile interface
				Storage.INFINITE_PAGE_POOL); // page pool is extended on demand
	}

	public void closeDatabase() {
		db.close();
	}

	public void initialize(IPersistent rootInstance) {
		IPersistent root = (IPersistent) db.getRoot();// get storage root
		if (root == null) {
			// Root is not yet defined: storage is not initialized
			db.setRoot(rootInstance); // register root object
		}
	}

	public void exportToXML(String xmlFileName) {
		try {
			Writer writer = new BufferedWriter(new FileWriter(xmlFileName));
			// export the whole database to the specified writer in XML
			// format
			db.exportXML(writer);
			// exportXML doesn't close the stream, close it here
			writer.close();
		} catch (IOException x) {
			System.err.println("Export failed: " + x);
		}
	}

	public void importFromXML(String xmlFileName) {
		try {
			Reader reader = new BufferedReader(new FileReader(xmlFileName));
			db.importXML(reader); // import data from XML stream
			// importXML doesn't close the stream, close it here
			reader.close();
		} catch (IOException x) {
			System.err.println("Import failed: " + x);
		} catch (XMLImportException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * @param arg0
	 * @throws IOException
	 * @see org.garret.perst.Storage#backup(java.io.OutputStream)
	 */
	public void backup(OutputStream arg0) throws IOException {
		db.backup(arg0);
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#beginSerializableTransaction()
	 */
	public void beginSerializableTransaction() {
		db.beginSerializableTransaction();
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#beginThreadTransaction(int)
	 */
	public void beginThreadTransaction(int arg0) {
		db.beginThreadTransaction(arg0);
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#clearObjectCache()
	 */
	public void clearObjectCache() {
		db.clearObjectCache();
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#close()
	 */
	public void close() {
		db.close();
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#commit()
	 */
	public void commit() {
		db.commit();
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#commitSerializableTransaction()
	 */
	public void commitSerializableTransaction() {
		db.commitSerializableTransaction();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createBitIndex()
	 */
	public BitIndex createBitIndex() {
		return db.createBitIndex();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @param arg3
	 * @return
	 * @see org.garret.perst.Storage#createBitmapAllocator(int, long, long,
	 *      long)
	 */
	public CustomAllocator createBitmapAllocator(int arg0, long arg1,
			long arg2, long arg3) {
		return db.createBitmapAllocator(arg0, arg1, arg2, arg3);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createBlob()
	 */
	public Blob createBlob() {
		return db.createBlob();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @param arg3
	 * @param arg4
	 * @return
	 * @see org.garret.perst.Storage#createFieldIndex(java.lang.Class,
	 *      java.lang.String, boolean, boolean, boolean)
	 */
	public FieldIndex createFieldIndex(Class arg0, String arg1, boolean arg2,
			boolean arg3, boolean arg4) {
		return db.createFieldIndex(arg0, arg1, arg2, arg3, arg4);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @param arg3
	 * @return
	 * @see org.garret.perst.Storage#createFieldIndex(java.lang.Class,
	 *      java.lang.String, boolean, boolean)
	 */
	public FieldIndex createFieldIndex(Class arg0, String arg1, boolean arg2,
			boolean arg3) {
		return db.createFieldIndex(arg0, arg1, arg2, arg3);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @return
	 * @see org.garret.perst.Storage#createFieldIndex(java.lang.Class,
	 *      java.lang.String, boolean)
	 */
	public FieldIndex createFieldIndex(Class arg0, String arg1, boolean arg2) {
		return db.createFieldIndex(arg0, arg1, arg2);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @param arg3
	 * @return
	 * @see org.garret.perst.Storage#createFieldIndex(java.lang.Class,
	 *      java.lang.String[], boolean, boolean)
	 */
	public FieldIndex createFieldIndex(Class arg0, String[] arg1, boolean arg2,
			boolean arg3) {
		return db.createFieldIndex(arg0, arg1, arg2, arg3);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @return
	 * @see org.garret.perst.Storage#createFieldIndex(java.lang.Class,
	 *      java.lang.String[], boolean)
	 */
	public FieldIndex createFieldIndex(Class arg0, String[] arg1, boolean arg2) {
		return db.createFieldIndex(arg0, arg1, arg2);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createFullTextIndex()
	 */
	public FullTextIndex createFullTextIndex() {
		return db.createFullTextIndex();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createFullTextIndex(org.garret.perst.fulltext.FullTextSearchHelper)
	 */
	public FullTextIndex createFullTextIndex(FullTextSearchHelper arg0) {
		return db.createFullTextIndex(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createHash()
	 */
	public IPersistentHash createHash() {
		return db.createHash();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createHash(int, int)
	 */
	public IPersistentHash createHash(int arg0, int arg1) {
		return db.createHash(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createIndex(java.lang.Class, boolean)
	 */
	public Index createIndex(Class arg0, boolean arg1) {
		return db.createIndex(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createIndex(java.lang.Class[], boolean)
	 */
	public Index createIndex(Class[] arg0, boolean arg1) {
		return db.createIndex(arg0, arg1);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createLink()
	 */
	public Link createLink() {
		return db.createLink();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createLink(int)
	 */
	public Link createLink(int arg0) {
		return db.createLink(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createList()
	 */
	public IPersistentList createList() {
		return db.createList();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createMap(java.lang.Class, int)
	 */
	public IPersistentMap createMap(Class arg0, int arg1) {
		return db.createMap(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createMap(java.lang.Class)
	 */
	public IPersistentMap createMap(Class arg0) {
		return db.createMap(arg0);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @return
	 * @see org.garret.perst.Storage#createMultidimensionalIndex(java.lang.Class,
	 *      java.lang.String[], boolean)
	 */
	public MultidimensionalIndex createMultidimensionalIndex(Class arg0,
			String[] arg1, boolean arg2) {
		return db.createMultidimensionalIndex(arg0, arg1, arg2);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createMultidimensionalIndex(org.garret.perst.MultidimensionalComparator)
	 */
	public MultidimensionalIndex createMultidimensionalIndex(
			MultidimensionalComparator arg0) {
		return db.createMultidimensionalIndex(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createPatriciaTrie()
	 */
	public PatriciaTrie createPatriciaTrie() {
		return db.createPatriciaTrie();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createQuery()
	 */
	public Query createQuery() {
		return db.createQuery();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessBlob()
	 */
	public Blob createRandomAccessBlob() {
		return db.createRandomAccessBlob();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @param arg3
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessFieldIndex(java.lang.Class,
	 *      java.lang.String, boolean, boolean)
	 */
	public FieldIndex createRandomAccessFieldIndex(Class arg0, String arg1,
			boolean arg2, boolean arg3) {
		return db.createRandomAccessFieldIndex(arg0, arg1, arg2, arg3);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessFieldIndex(java.lang.Class,
	 *      java.lang.String, boolean)
	 */
	public FieldIndex createRandomAccessFieldIndex(Class arg0, String arg1,
			boolean arg2) {
		return db.createRandomAccessFieldIndex(arg0, arg1, arg2);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @param arg3
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessFieldIndex(java.lang.Class,
	 *      java.lang.String[], boolean, boolean)
	 */
	public FieldIndex createRandomAccessFieldIndex(Class arg0, String[] arg1,
			boolean arg2, boolean arg3) {
		return db.createRandomAccessFieldIndex(arg0, arg1, arg2, arg3);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessFieldIndex(java.lang.Class,
	 *      java.lang.String[], boolean)
	 */
	public FieldIndex createRandomAccessFieldIndex(Class arg0, String[] arg1,
			boolean arg2) {
		return db.createRandomAccessFieldIndex(arg0, arg1, arg2);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessIndex(java.lang.Class,
	 *      boolean)
	 */
	public Index createRandomAccessIndex(Class arg0, boolean arg1) {
		return db.createRandomAccessIndex(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createRandomAccessIndex(java.lang.Class[],
	 *      boolean)
	 */
	public Index createRandomAccessIndex(Class[] arg0, boolean arg1) {
		return db.createRandomAccessIndex(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createRelation(java.lang.Object)
	 */
	public Relation createRelation(Object arg0) {
		return db.createRelation(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createScalableList()
	 */
	public IPersistentList createScalableList() {
		return db.createScalableList();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createScalableList(int)
	 */
	public IPersistentList createScalableList(int arg0) {
		return db.createScalableList(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createScalableSet()
	 */
	public IPersistentSet createScalableSet() {
		return db.createScalableSet();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createScalableSet(int)
	 */
	public IPersistentSet createScalableSet(int arg0) {
		return db.createScalableSet(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createSet()
	 */
	public IPersistentSet createSet() {
		return db.createSet();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createSortedCollection(org.garret.perst.PersistentComparator,
	 *      boolean)
	 */
	public SortedCollection createSortedCollection(PersistentComparator arg0,
			boolean arg1) {
		return db.createSortedCollection(arg0, arg1);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createSpatialIndex()
	 */
	public SpatialIndex createSpatialIndex() {
		return db.createSpatialIndex();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#createSpatialIndexR2()
	 */
	public SpatialIndexR2 createSpatialIndexR2() {
		return db.createSpatialIndexR2();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#createThickIndex(java.lang.Class)
	 */
	public Index createThickIndex(Class arg0) {
		return db.createThickIndex(arg0);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#createTimeSeries(java.lang.Class, long)
	 */
	public TimeSeries createTimeSeries(Class arg0, long arg1) {
		return db.createTimeSeries(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#deallocate(java.lang.Object)
	 */
	public void deallocate(Object arg0) {
		db.deallocate(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#deallocateObject(java.lang.Object)
	 */
	public void deallocateObject(Object arg0) {
		db.deallocateObject(arg0);
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#endThreadTransaction()
	 */
	public void endThreadTransaction() {
		db.endThreadTransaction();
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#endThreadTransaction(int)
	 */
	public void endThreadTransaction(int arg0) {
		db.endThreadTransaction(arg0);
	}

	/**
	 * @param arg0
	 * @throws IOException
	 * @see org.garret.perst.Storage#exportXML(java.io.Writer)
	 */
	public void exportXML(Writer arg0) throws IOException {
		db.exportXML(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#findClassLoader(java.lang.String)
	 */
	public ClassLoader findClassLoader(String arg0) {
		return db.findClassLoader(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#gc()
	 */
	public int gc() {
		return db.gc();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getClassLoader()
	 */
	public ClassLoader getClassLoader() {
		return db.getClassLoader();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getDatabaseFormatVersion()
	 */
	public int getDatabaseFormatVersion() {
		return db.getDatabaseFormatVersion();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getDatabaseSize()
	 */
	public long getDatabaseSize() {
		return db.getDatabaseSize();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getListener()
	 */
	public StorageListener getListener() {
		return db.getListener();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getMemoryDump()
	 */
	public HashMap getMemoryDump() {
		return db.getMemoryDump();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#getObjectByOID(int)
	 */
	public Object getObjectByOID(int arg0) {
		return db.getObjectByOID(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#getOid(java.lang.Object)
	 */
	public int getOid(Object arg0) {
		return db.getOid(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getProperties()
	 */
	public Properties getProperties() {
		return db.getProperties();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#getProperty(java.lang.String)
	 */
	public Object getProperty(String arg0) {
		return db.getProperty(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getRoot()
	 */
	public Object getRoot() {
		return db.getRoot();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getSqlOptimizerParameters()
	 */
	public SqlOptimizerParameters getSqlOptimizerParameters() {
		return db.getSqlOptimizerParameters();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getTransactionContext()
	 */
	public ThreadTransactionContext getTransactionContext() {
		return db.getTransactionContext();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#getUsedSize()
	 */
	public long getUsedSize() {
		return db.getUsedSize();
	}

	/**
	 * @param arg0
	 * @throws XMLImportException
	 * @see org.garret.perst.Storage#importXML(java.io.Reader)
	 */
	public void importXML(Reader arg0) throws XMLImportException {
		db.importXML(arg0);
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#isInsideThreadTransaction()
	 */
	public boolean isInsideThreadTransaction() {
		return db.isInsideThreadTransaction();
	}

	/**
	 * @return
	 * @see org.garret.perst.Storage#isOpened()
	 */
	public boolean isOpened() {
		return db.isOpened();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#join(java.util.Iterator[])
	 */
	public Iterator join(Iterator[] arg0) {
		return db.join(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#load(java.lang.Object)
	 */
	public void load(Object arg0) {
		db.load(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#loadObject(java.lang.Object)
	 */
	public void loadObject(Object arg0) {
		db.loadObject(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#lockObject(java.lang.Object)
	 */
	public boolean lockObject(Object arg0) {
		return db.lockObject(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#makePersistent(java.lang.Object)
	 */
	public int makePersistent(Object arg0) {
		return db.makePersistent(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#merge(java.util.Iterator[])
	 */
	public Iterator merge(Iterator[] arg0) {
		return db.merge(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#modify(java.lang.Object)
	 */
	public void modify(Object arg0) {
		db.modify(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#modifyObject(java.lang.Object)
	 */
	public void modifyObject(Object arg0) {
		db.modifyObject(arg0);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @see org.garret.perst.Storage#open(org.garret.perst.IFile, long)
	 */
	public void open(IFile arg0, long arg1) {
		db.open(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#open(org.garret.perst.IFile)
	 */
	public void open(IFile arg0) {
		db.open(arg0);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param arg2
	 * @see org.garret.perst.Storage#open(java.lang.String, long,
	 *      java.lang.String)
	 */
	public void open(String arg0, long arg1, String arg2) {
		db.open(arg0, arg1, arg2);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @see org.garret.perst.Storage#open(java.lang.String, long)
	 */
	public void open(String arg0, long arg1) {
		db.open(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#open(java.lang.String)
	 */
	public void open(String arg0) {
		db.open(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#registerClassLoader(org.garret.perst.INamedClassLoader)
	 */
	public void registerClassLoader(INamedClassLoader arg0) {
		db.registerClassLoader(arg0);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @see org.garret.perst.Storage#registerCustomAllocator(java.lang.Class,
	 *      org.garret.perst.CustomAllocator)
	 */
	public void registerCustomAllocator(Class arg0, CustomAllocator arg1) {
		db.registerCustomAllocator(arg0, arg1);
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#rollback()
	 */
	public void rollback() {
		db.rollback();
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#rollbackSerializableTransaction()
	 */
	public void rollbackSerializableTransaction() {
		db.rollbackSerializableTransaction();
	}

	/**
	 * 
	 * @see org.garret.perst.Storage#rollbackThreadTransaction()
	 */
	public void rollbackThreadTransaction() {
		db.rollbackThreadTransaction();
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#setClassLoader(java.lang.ClassLoader)
	 */
	public ClassLoader setClassLoader(ClassLoader arg0) {
		return db.setClassLoader(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#setCustomSerializer(org.garret.perst.CustomSerializer)
	 */
	public void setCustomSerializer(CustomSerializer arg0) {
		db.setCustomSerializer(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#setGcThreshold(long)
	 */
	public void setGcThreshold(long arg0) {
		db.setGcThreshold(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#setListener(org.garret.perst.StorageListener)
	 */
	public StorageListener setListener(StorageListener arg0) {
		return db.setListener(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#setProperties(java.util.Properties)
	 */
	public void setProperties(Properties arg0) {
		db.setProperties(arg0);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @see org.garret.perst.Storage#setProperty(java.lang.String,
	 *      java.lang.Object)
	 */
	public void setProperty(String arg0, Object arg1) {
		db.setProperty(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @return
	 * @see org.garret.perst.Storage#setRecursiveLoading(java.lang.Class,
	 *      boolean)
	 */
	public boolean setRecursiveLoading(Class arg0, boolean arg1) {
		return db.setRecursiveLoading(arg0, arg1);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#setRoot(java.lang.Object)
	 */
	public void setRoot(Object arg0) {
		db.setRoot(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see org.garret.perst.Storage#setTransactionContext(org.garret.perst.impl.ThreadTransactionContext)
	 */
	public ThreadTransactionContext setTransactionContext(
			ThreadTransactionContext arg0) {
		return db.setTransactionContext(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#store(java.lang.Object)
	 */
	public void store(Object arg0) {
		db.store(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#storeFinalizedObject(java.lang.Object)
	 */
	public void storeFinalizedObject(Object arg0) {
		db.storeFinalizedObject(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#storeObject(java.lang.Object)
	 */
	public void storeObject(Object arg0) {
		db.storeObject(arg0);
	}

	/**
	 * @param arg0
	 * @see org.garret.perst.Storage#throwObject(java.lang.Object)
	 */
	public void throwObject(Object arg0) {
		db.throwObject(arg0);
	}

	// (Storage.SERIALIZABLE_TRANSACTION)
	// void run() {
	// Storage db = obj1.getStorage();
	// // start serializable transaction
	// db.beginThreadTransaction(Storage.SERIALIZABLE_TRANSACTION);
	// try {
	// obj1.someReadOnlyMethod();
	// obj2.someUpdateMethod();
	// // commit changes in case of normal completion
	// db.endThreadTransaction();
	// } catch (Exception x) {
	// // rollback transaction in case of exception
	// db.rollbackThreadTransaction();
	// }
	// }
	//
	// (Storage.EXCLUSIVE_TRANSACTION)
	//
	// db.beginThreadTransaction(Storage.EXCLUSIVE_TRANSACTION);
	// try {
	// // do something
	// // commit changes in case of normal completion
	// db.endThreadTransaction();
	// } catch (Exception x) {
	// // rollback transaction in case of exception
	// db.rollbackThreadTransaction();
	// }
}
